/***************************************************************************
 *   Copyright (C) 2006 by Gustavo Pichorim Boiko                          *
 *   boiko@mandriva.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "menuwidget.h"
#include "menumanager.h"
#include "theme.h"
#include "firstlevelmenu.h"
#include "secondlevelmenu.h"
#include "searchbar.h"

#include <qlayout.h>
#include <qpixmap.h>

#include <kdebug.h>
#include <kstandarddirs.h>
#include <kglobal.h>
#include <kconfig.h>
#include <kurl.h>

using namespace Menu;
MenuWidget::MenuWidget(QWidget *parent, const char *name)
 : QWidget(parent, name, 
           WStyle_Customize | /*WRepaintNoErase |*/ WStyle_NoBorder |
                   WDestructiveClose | WStyle_StaysOnTop /*| Qt::WPaintDesktop*/)
{
    m_menuManager = new Menu::Manager(this);
    
    KConfig cfg("mandrivamenurc");
    cfg.setGroup("Theme");
    applyTheme(cfg.readEntry("ThemeName", "default"));
}


MenuWidget::~MenuWidget()
{
}

void MenuWidget::addObject(ThemeObject *object)
{
    QWidget *widget = dynamic_cast<QWidget*>(object);
    if (widget)
    {
        //widget->reparent(this);
        widget->reparent(this, widget->rect().topLeft(), true);
        widget->move(object->position());
        //widget->hide();
        QRect r = widget->rect();
        r.moveTopLeft(object->position());
        widget->setBackgroundPixmap( Theme::createBackground(m_objectList, r, this));
        widget->show();
    }
    m_objectList.insert(object->objectName(), object);
}

void MenuWidget::applyTheme(const QString &t)
{
    QString theme;
    KURL path;
    if (t.isNull())
        theme = QString("default");
    else
        theme = t;
    path = locate("data", "mandrivamenu/themes/" + theme + "/themerc");
    
    if (path.isEmpty())
    {
        // Try to parse the theme as a KBFX theme
        applyKbfxTheme(t);
        return;
    }
    
    KConfig themerc(path.path());
    themerc.setGroup("Theme");
    
    // resize the main widget
    resize(themerc.readSizeEntry("Size", &QSize(300, 400)));
    setMinimumSize(size());
    QString background = themerc.readEntry("Background", QString::null);
    kdDebug() << path.directory() + "/" + background << endl;
    if (!background.isNull())
    {
        QPixmap bg(path.directory() + "/" + background);
        setBackgroundPixmap(bg);
        if (bg.hasAlpha())
        {
            m_mask = *bg.mask();
            setMask(m_mask);
        }
    }
    
    // read objects
    QStringList objects = themerc.readListEntry("Objects");
    
    QStringList::Iterator it;
    ThemeObject *obj;
    for (it = objects.begin(); it != objects.end(); ++it)
    {
        obj = Theme::createObject(&themerc, path.directory(), (*it));
        if (obj)
            addObject(obj);
    }
    
    prepareObjects();
}

void MenuWidget::applyKbfxTheme(const QString &t)
{
    /**
     * Kbfx themes are composed by fixed components
     *    - topbg.png
     *    - botbg.png
     *    - bg.png
     *    - tabbg.png
     *    - listboxbg.png
     *    - tilenormal.png
     *    - tilehover.png
     *    - rhstile.png
     *    - rhshovertile.png
     *    - logout.png
     *    - logouthover.png
     *    - lock.png
     *    - logout.png
     *    - mask.png
     */
    QString theme = locate("data", "mandrivamenu/themes/" + t + "/bg.png");
    
    if (!theme)
        return;
    setBackgroundPixmap(QPixmap(theme));
    theme = theme.remove("/bg.png");
    
    QPixmap mask(theme + "/mask.png");
    if (!mask.isNull())
    {
        resize(mask.size());
        setMinimumSize(mask.size());
        if (mask.hasAlpha())
        {
            m_mask = *mask.mask();
            setMask(m_mask);
        }
    }
    
    KConfig themerc(locate("data", "mandrivamenu/kbfxrc"));
    themerc.setGroup("Theme");
    // read objects
    QStringList objects = themerc.readListEntry("Objects");
    
    QStringList::Iterator it;
    
    /**
     * For calculating purposes, we have a QRect for KBFX objects
     * 
     * We assume topbg will be loaded always before botbg
     */
    QRect topbg, botbg, menu, lock, logout, findimg;
    
    ThemeObject *obj;
    for (it = objects.begin(); it != objects.end(); ++it)
    {
        obj = Theme::createObject(&themerc, theme, (*it));
        if (!obj)
            continue;
        
        // All ThemeObjects are QWidget based classes
        QWidget *w = dynamic_cast<QWidget*>(obj);
        if ((*it) == "topbg")
        {
            obj->setPosition(QPoint(0,0));
            topbg = (dynamic_cast<QWidget*>(obj))->rect();
        }
        else if ((*it) == "user")
        {
            obj->setPosition(QPoint((topbg.height() - w->height())/2, 
                             (topbg.height() - w->height())/2));
        }
        else if ((*it) == "username")
        {
            obj->setPosition(QPoint(50,10));
        }
        else if ((*it) == "botbg")
        {
            obj->setPosition(QPoint(0, height() - w->height()));
            botbg = QRect(0,height() - w->height(), 
                          w->width(), w->height());
        }
        else if ((*it) == "menu")
        {
            // FIXME: do not open the png to discover the height
            QPixmap p(theme + "/rhstile.png");
            obj->setPosition(QPoint(0, topbg.height()));
            w->resize(p.width(), 
                      height() - topbg.height() - botbg.height());
            menu = w->rect();
        }
        else if ((*it) == "menu2")
        {
            obj->setPosition(QPoint(menu.width(), topbg.height()));
            w->resize(width() - menu.width(),
                      height() - topbg.height() - botbg.height()); 
        }
        else if ((*it) == "lock")
        {
            obj->setPosition(QPoint(width() - w->width(), height() - w->height()));
            lock = w->rect();
        }
        else if ((*it) == "logout")
        {
            obj->setPosition(QPoint(width() - w->width() - lock.width(), height() - w->height()));
            logout = w->rect();
        }
        else if ((*it) == "findimg")
        {
            QPixmap p(theme + "/find.png");
            obj->setPosition(QPoint(botbg.left() + 5, botbg.top() + (botbg.height() - p.height())/2));
            findimg = p.rect();
        }
        else if ((*it) == "find")
        {
            QPixmap p(theme + "/appfind.png");
            
            obj->setPosition(QPoint(botbg.left() + findimg.width() + 5,botbg.top() + (botbg.height() - p.height())/2));
            w->resize(width() - lock.width() - logout.width() - findimg.width() - 10, p.height());
        }
        
        QRect r = w->rect();
        r.moveTopLeft(obj->position());
        addObject(obj);
    }
    
    prepareObjects();
}

void MenuWidget::prepareObjects()
{
    // Find the first and second level menus
    // for connecting them 
    FirstLevelMenu *fmenu = dynamic_cast<FirstLevelMenu*>(
            Theme::findObject(m_objectList, 
                              Theme::FirstLevelMenuObj));
    if (!fmenu)
        return;
    
    SecondLevelMenu *smenu = dynamic_cast<SecondLevelMenu*>(
            Theme::findObject(m_objectList, 
                              Theme::SecondLevelMenuObj));
    
    if (!smenu)
        return;
    
    fmenu->listServices(m_menuManager->rootMenu());
    
    connect(fmenu, SIGNAL(serviceGroupSelected(const QString&)),
            m_menuManager, SLOT(slotListMenu(const QString&)));
    
    connect(m_menuManager, SIGNAL(listMenuEntries(const EntryList&)),
            smenu, SLOT(listMenuEntries(const EntryList&)));
    
    // Search bar
    SearchBar *bar = dynamic_cast<SearchBar*>(
            Theme::findObject(m_objectList,
                              Theme::SearchBarObj));
    
    if (!bar)
        return;
    
    connect(bar, SIGNAL(search(const QString&)),
            m_menuManager, SLOT(slotSearch(const QString&)));
    
    connect(fmenu, SIGNAL(serviceGroupSelected(const QString&)),
            bar, SLOT(slotClear()));
    
    connect(m_menuManager, SIGNAL(searching(bool)),
            fmenu, SLOT(slotSearching(bool)));
    kdDebug() << "Finished preparing all" << endl;
}

#include "menuwidget.moc"
